#include "WiFiWebserver.h"

// we use the WiFiServer and WiFiClient functions from Arduino
#include <WiFi.h>

WiFiClient myclient;
WiFiServer server(80);


// Constructor of Class WiFiWebserver   (function name = class name)
WiFiWebserver::WiFiWebserver()
{
	
}



// start the webserver
void WiFiWebserver::Start()
{
  server.begin();     // Start TCP/IP-Server
}



//  Call this function regularly to look for client requests
//  template see https://github.com/espressif/arduino-esp32/blob/master/libraries/WiFi/examples/SimpleWiFiServer/SimpleWiFiServer.ino
//  returns empty string if no request from any client
//  returns GET Parameter: everything after the "/?" if ADDRESS/?xxxx was entered by the user in the webbrowser
//  returns "-" if ADDRESS but no GET Parameter was entered by the user in the webbrowser
//  remark: client connection stays open after return

String WiFiWebserver::GetRequestGETParameter()
{
  String GETParameter = "";
  WebRequestHostPage  = "";
  
  myclient = server.available();             // listen for incoming clients

  
  if (myclient) {                            // if you get a client,
    Serial.println("New Client.");           // print a message out the serial port
    String currentLine = "";                 // make a String to hold incoming data from the client

    unsigned int ConnectedCount = 0;
    
    while (myclient.connected()) {           // loop while the client's connected

      ConnectedCount++;                      // if connected but no data available

      if (ConnectedCount > 5)                // after 5 times we break out of the loop
      {
        break;
      }
      
      if (myclient.available()) {            // if there are bytes to read from the client

        ConnectedCount = 0;
        
        char c = myclient.read();            // read a byte, then
        Serial.write(c);                     // print it out the serial monitor

        if (c == '\n') {                     // if the byte is a newline character

          // if the current line is blank, you got two newline characters in a row.
          // that's the end of the client HTTP request
          if (currentLine.length() == 0) {
            
            if (GETParameter == "") {GETParameter = "-";};    // if no "GET /?" was found so far in the request bytes, return "-"
            
            // break out of the while loop
            
            break;
        
          } else {    // if you got a newline, then clear currentLine:
            currentLine = "";
          }
          
        } else if (c != '\r') {  // if you got anything else but a carriage return character,
          currentLine += c;      // add it to the end of the currentLine
        }

        if ( c=='\r' && currentLine.startsWith("GET /") ) 
        // we see a "GET /" in the HTTP data of the client request
        // user entered ADDRESS/?xxxx     in webbrowser, xxxx = GET Parameter
	// user entered ADDRESS/PAGE?xxxx in webbrowser, xxxx = GET Parameter, /PAGE = WebRequestHostPage
        {

		String HostPage = "";
		int posOfQuestionMark = currentLine.indexOf('?');

		if (posOfQuestionMark > -1)  // a question mark was found, so user entered some GET Parameter
		{	         
          		HostPage =        currentLine.substring(currentLine.indexOf(' ') + 1, posOfQuestionMark);    // extract everything behind the first space and before the question mark
			GETParameter =    currentLine.substring(posOfQuestionMark + 1, currentLine.indexOf(' ', 6));    // extract everything behind the question mark and before the next space
		}
		else
		{
			HostPage =        currentLine.substring(currentLine.indexOf(' ') + 1, currentLine.indexOf(' ', 6));    // extract everything behind the first and next space
		}

		if (HostPage != "/favicon.ico")   // omit the favicon ico get request
		{
			WebRequestHostPage = HostPage;
		}
                   
        }


        if (c=='\r' && currentLine.startsWith("Host:")) 
        // we see a "Host:" in the HTTP data of the client request
        // user entered ADDRESS or ADDRESS/... in webbrowser, ADDRESS = Server IP-Address of HTTP-Request
        {
        	int IndexOfBlank = currentLine.indexOf(' ');
          	WebRequestHostAddress = currentLine.substring(IndexOfBlank + 1, currentLine.length());    // extract everything behind the space character and store Server IP-Address of HTTP-Request
          
        }
        
      }
      
    }
    
  }

  return GETParameter;
}


// decodes Url-Escape-Codes in String
String WiFiWebserver::DecodeURLChars(String strURL)
{
  strURL.replace("+", " ");
  strURL.replace("%24", "$");
  strURL.replace("%2F", "/");
  strURL.replace("%3A", ":");
  strURL.replace("%21", "!");
  strURL.replace("%3F", "?");
  strURL.replace("%23", "#");
  strURL.replace("%40", "@");
  strURL.replace("%7B", "{");
  strURL.replace("%7D", "}");
  strURL.replace("%22", String(char('\"')));
  return strURL;
}



// extract key values from GET Parameter Expression
// name1=Value1&name2=value2&name3=value3 and so on
// pls note: no check of the name/keys !
Param WiFiWebserver::DeSerializeFromGETParameter(String GETParameter, Param ResultParam)
{
   int posFirstCharToSearch = 1;
   int count = 0;
   
   // while a "&" is in the expression, after a start position to search
   while (GETParameter.indexOf('&', posFirstCharToSearch) > -1)
   {
     int posOfSeparatorChar = GETParameter.indexOf('&', posFirstCharToSearch);  // position of & after start position
     int posOfValueChar = GETParameter.indexOf('=', posFirstCharToSearch);      // position of = after start position
  
     String strChars = GETParameter.substring(posOfValueChar + 1, posOfSeparatorChar);  // extract everything between = and &
     
     ResultParam.PValue[count] = DecodeURLChars(strChars);  // decode the escape chars in the extracted string and put it as value in the ResultParam parameter set

     posFirstCharToSearch = posOfSeparatorChar + 1;  // shift the start position to search after the &-char 
     count++;
   }

   // no more &-chars found
   
   int posOfValueChar = GETParameter.indexOf('=', posFirstCharToSearch);       // search for =
   
   String strChars = GETParameter.substring(posOfValueChar + 1, GETParameter.length());  // extract everything between = and end of string
   
   ResultParam.PValue[count] = DecodeURLChars(strChars);  // decode the escape chars in the extracted string and put it as last value in the ResultParam parameter set 
   
   count++;

   ResultParam.PCount = count;  // set the number of set values of the ResultParam parameter set 

   return ResultParam;  // number of values found in GET Parameter Expression
}




// make a HTML table from a Param Key Value Set
// for each Value in the Param:
// first table column will be filled with Name/Key
// second table column will be filled with a control to set the Value
String WiFiWebserver::SerializeToHTMLInputTable(Param TableParam)
{
   String HTMLResult = "<table>";  // start tag for HTML table	

   // for each Param value
   for (int c = 0; c < TableParam.PCount; c++)
   {
      // set background color by the status of the Param value
      String StyleHTML = "";
      if (TableParam.PStatus[c] == 0) { StyleHTML = " Style =\"background-color: #FFE4B5;\" " ;};   // yellow  for status not set
      if (TableParam.PStatus[c] == 1) { StyleHTML = " Style =\"background-color: #98FB98;\" " ;};   // green   for status valid
      if (TableParam.PStatus[c] == -1) { StyleHTML = " Style =\"background-color: #FA8072;\" " ;};  // red     for status invalid

      String TableRowHTML = "";
  
      if (TableParam.PType[c] == 1)    //  string value, entered in text box
      {
        // build the HTML code for a table row with Param value name and the value itself inside a textbox   
        TableRowHTML = "<tr><th>" + TableParam.PName[c] + "</th><th><input name=\"" + TableParam.PName[c] + "\" value=\"" + TableParam.PValue[c] + "\" " + StyleHTML + " /></th></tr>";
      }
  
      if (TableParam.PType[c] == 2)   // byte value, to be set by slider
      {
        // build the HTML code for a table row with Param value name and the byte value represented by a slider 
        TableRowHTML = "<tr><th>" + TableParam.PName[c] + "</th><th><input type=\"range\" min=\"1\"  max=\"255\" name=\"" + TableParam.PName[c] + "\" value=\"" + TableParam.PValue[c] + "\" " + StyleHTML + " /></th></tr>";
      }
      
      // add the table row HTML code to the page
      HTMLResult += TableRowHTML;
   }

   HTMLResult += "</table>";      // end tag for HTML table	

   return HTMLResult;

}



// Build a HTML page with a form 
// InnerHTML is the code of the form with the HTML controls (textboxes, sliders and so on)
// returns the HTML code of the page
String WiFiWebserver::FormHTML(String TitleOfForm, String InnerHTML)
{
   // Head of the HTML page
   String HTMLPage = "<!DOCTYPE html><html><body><h2>" + TitleOfForm + "</h2><form>";

   HTMLPage += InnerHTML;  // add the code of the form with the HTML controls (textboxes, sliders and so on)

   // add the submit button
   HTMLPage += "<br/><input type=\"submit\" value=\"Submit\" />";

   // footer of the webpage
   HTMLPage += "</form></body></html>";
   
   return HTMLPage;
}
		




// Send HTML page to client, as HTTP response
// client connection must be open (call Webserver_GetRequestGETParameter() first)
void WiFiWebserver::SendHTMLPage(String HTMLPage)
{
   String httpResponse = "";

   // begin with HTTP response header
   httpResponse += "HTTP/1.1 200 OK\r\n";
   httpResponse += "Content-type:text/html\r\n\r\n";

   // then the HTML page
   httpResponse += HTMLPage;

   // The HTTP response ends with a blank line:
   httpResponse += "\r\n";
    
   // send it out to TCP/IP client (webbrowser) 
   myclient.println(httpResponse);

   // close the connection
   myclient.stop();
       
}

;